@@ -10,6 +10,7 @@
 
 //#include<iosfwd>
 #include<string>
+#include <vector>
 
 #include "commands/basecommand.h"
 
@@ -73,6 +74,7 @@ struct ZypperCommand
   static const ZypperCommand PS;
   static const ZypperCommand DOWNLOAD;
   static const ZypperCommand SOURCE_DOWNLOAD;
+  static const ZypperCommand PURGE_KERNELS;
 
   static const ZypperCommand HELP;
   static const ZypperCommand SHELL;
@@ -85,17 +87,14 @@ struct ZypperCommand
   static const ZypperCommand RUG_PATCH_INFO;
   static const ZypperCommand RUG_PATTERN_INFO;
   static const ZypperCommand RUG_PRODUCT_INFO;
-  static const ZypperCommand RUG_SERVICE_TYPES;
-  static const ZypperCommand RUG_LIST_RESOLVABLES;
-  static const ZypperCommand RUG_MOUNT;
-  //static const ZypperCommand RUG_INFO_PROVIDES;
-  //static const ZypperCommand RUG_INFO_CONFLICTS;
-  //static const ZypperCommand RUG_INFO_OBSOLETES;
-  //static const ZypperCommand RUG_INFO_REQUIREMENTS;
-  static const ZypperCommand RUG_PATCH_SEARCH;
-  static const ZypperCommand RUG_PING;
   //!@}
 
+  static const ZypperCommand NEEDS_REBOOTING;
+
+  static const ZypperCommand LOCALES;
+  static const ZypperCommand ADD_LOCALE;
+  static const ZypperCommand REMOVE_LOCALE;
+
   enum Command
   {
     NONE_e,
@@ -150,6 +149,7 @@ struct ZypperCommand
     PS_e,
     DOWNLOAD_e,
     SOURCE_DOWNLOAD_e,
+    PURGE_KERNELS_e,
 
     HELP_e,
     SHELL_e,
@@ -161,19 +161,45 @@ struct ZypperCommand
     RUG_PATCH_INFO_e,
     RUG_PATTERN_INFO_e,
     RUG_PRODUCT_INFO_e,
-    RUG_PATCH_SEARCH_e,
-    RUG_PING_e
+
+    LOCALES_e,
+    ADD_LOCALE_e,
+    REMOVE_LOCALE_e,
+
+    NEEDS_REBOOTING_e
   };
 
+  using CmdFactory = std::function<ZypperBaseCommandPtr ()>;
+
+  enum CmdDescField : int
+  {
+    Id = 0,
+    Category,
+    Alias,
+    Factory
+  };
+  using CmdDesc = std::tuple< ZypperCommand::Command, std::string, std::vector<const char *>, CmdFactory >;
+
   ZypperCommand( Command command );
 
   explicit ZypperCommand( const std::string & strval_r );
 
   Command toEnum() const { return _command; }
 
+  static Command toEnum (const std::string & strval_r );
+
   const std::string & asString() const;
 
   ZypperBaseCommandPtr commandObject() const;
+  ZypperBaseCommand& assertCommandObject() const;
+
+  template<typename T>
+  T& assertCommandObject () const
+  {
+    return dynamic_cast<T &>( assertCommandObject() );
+  }
+
+  static const std::vector<CmdDesc> &allCommands ();
 
 private:
   /** Fills SubcommandOptions::Detected on the fly if SUBCOMMAND */
@@ -181,7 +207,8 @@ private:
   Command _command;
 
   /** Only initialized if we have a new style command */
-  ZypperBaseCommandPtr _newStyleCmdObj;
+  //mutable to support late initializing when the command object is requested for the first time
+  mutable ZypperBaseCommandPtr _newStyleCmdObj;
 
 };
 
@@ -191,6 +218,12 @@ inline std::ostream & operator<<( std::ostream & str, const ZypperCommand & obj
 inline bool operator==(const ZypperCommand & obj1, const ZypperCommand & obj2)
 { return obj1.toEnum() == obj2.toEnum(); }
 
+inline bool operator==(const ZypperCommand & obj1, const ZypperCommand::Command & enu)
+{ return obj1.toEnum() == enu; }
+
+inline bool operator==(const ZypperCommand::Command & enu, const ZypperCommand & obj )
+{ return obj.toEnum() == enu; }
+
 inline bool operator!=(const ZypperCommand & obj1, const ZypperCommand & obj2)
 { return obj1.toEnum() != obj2.toEnum(); }
 
