@@ -22,6 +22,7 @@
 
 #include "Zypper.h"
 #include "SolverRequester.h"
+#include "global-settings.h"
 
 // libzypp logger settings
 #undef  ZYPP_BASE_LOGGER_LOGGROUP
@@ -230,7 +231,7 @@ void SolverRequester::install( const PackageSpec & pkg )
     // get the best matching items and tag them for installation.
     // FIXME this ignores vendor lock - we need some way to do --from which
     // would respect vendor lock: e.g. a new Selectable::updateCandidateObj(Options&)
-    PoolItemBest bestMatches( q.begin(), q.end() );
+    PoolItemBest bestMatches( q.begin(), q.end(), PoolItemBest::preferNotLocked );
 
     if ( !bestMatches.empty() )
     {
@@ -263,12 +264,20 @@ void SolverRequester::install( const PackageSpec & pkg )
             // check vendor (since PoolItemBest does not do it)
             bool changes_vendor = ! VendorAttr::instance().equivalent( instobj->vendor(), (*sit)->vendor() );
 
-            PoolItem best;
+            PoolItem best { s->updateCandidateObj() };
+	    if ( best && best.status().isLocked()
+	      && !(*sit).status().isLocked()
+	      && (*sit).edition() > instobj.edition() )
+	    {
+	      // This is a partially locked item. We try the best unlocked version.
+	      best = PoolItem();
+	    }
+
             if ( userconstraints )
               updateTo( pkg, *sit);
             else if  (_opts.force )
               updateTo( pkg, s->highestAvailableVersionObj() );
-            else if ( (best = s->updateCandidateObj()) )
+            else if ( best )
               updateTo( pkg, best );
             else if ( changes_vendor && !_opts.allow_vendor_change )
               updateTo( pkg, instobj );
@@ -429,7 +438,7 @@ void SolverRequester::updatePatterns()
 
 // ----------------------------------------------------------------------------
 
-void SolverRequester::updatePatches()
+void SolverRequester::updatePatches( bool updateStackOnly )
 {
   DBG << "going to mark needed patches for installation" << endl;
 
@@ -481,7 +490,7 @@ void SolverRequester::updatePatches()
 	}
       }
 
-      if ( Zypper::instance().cOpts().count("updatestack-only") )
+      if ( updateStackOnly )
       {
 	MIL << "updatestack-only: will stop here!" << endl;
 	break;
@@ -516,9 +525,9 @@ bool SolverRequester::installPatch( const PackageSpec & patchspec, const PoolIte
     if ( ignore_pkgmgmt || patch->restartSuggested() )
     {
       Patch::InteractiveFlags ignoreFlags = Patch::NoFlags;
-      if ( Zypper::instance().globalOpts().reboot_req_non_interactive )
+      if ( Zypper::instance().config().reboot_req_non_interactive )
         ignoreFlags |= Patch::Reboot;
-      if ( Zypper::instance().cOpts().count("auto-agree-with-licenses") )
+      if ( LicenseAgreementPolicy::instance()._autoAgreeWithLicenses )
 	ignoreFlags |= Patch::License;
 
       if ( selected.isUnwanted() )
@@ -728,8 +737,8 @@ void SolverRequester::updateTo( const PackageSpec & pkg, const PoolItem & select
       DBG << "Newer object exists, but has different repo/arch/version: " << highest << endl;
     }
 
-    // update candidate locked
-    if ( s->status() == ui::S_Protected || highest.status().isLocked() )
+    // update candidate locked (suppress if selected is not locked)
+    if ( selected.status().isLocked() && ( s->status() == ui::S_Protected || highest.status().isLocked() ) )
     {
       addFeedback( Feedback::UPD_CANDIDATE_IS_LOCKED, pkg, selected, installed );
       DBG << "Newer object exists, but is locked: " << highest << endl;
@@ -763,7 +772,7 @@ void SolverRequester::setToInstall( const PoolItem & pi )
     pi.status().setToBeInstalled( ResStatus::USER );
     addFeedback( Feedback::FORCED_INSTALL, PackageSpec(), pi );
   }
-  else if ( ui::asSelectable()(pi)->locked() )
+  else if ( ui::asSelectable()(pi)->hasLocks() )
   {
     // Workaround: Use a solver request instead of selecting the item.
     // This will enable the solver to report the lock conflict, while
@@ -790,7 +799,7 @@ void SolverRequester::setToInstall( const PoolItem & pi )
 
 void SolverRequester::setToRemove( const PoolItem & pi )
 {
-  if ( ui::asSelectable()(pi)->locked() )
+  if ( ui::asSelectable()(pi)->hasLocks() )
   {
     // Workaround: Use a solver request instead of selecting the item.
     // This will enable the solver to report the lock conflict, while
