@@ -76,7 +76,8 @@ typedef std::map<Resolvable::Kind, std::set<ResObject::constPtr, ResNameCompare>
 void Summary::readPool( const ResPool & pool )
 {
   // reset stats
-  _need_reboot = false;
+  _need_reboot_patch = false;
+  _need_reboot_nonpatch = false;
   _need_restart = false;
   _inst_pkg_total = 0;
 
@@ -115,8 +116,8 @@ void Summary::readPool( const ResPool & pool )
         // set the 'need reboot' flag
         if ( patch->rebootSuggested() )
 	{
-          _need_reboot = true;
-	  _rebootNeeded.insert( ResPair( nullptr, patch ) );
+          _need_reboot_patch = true;
+	  _rebootNeeded[ResKind::patch].insert( ResPair( nullptr, patch ) );
 	}
         else if ( patch->restartSuggested() )
           _need_restart = true;
@@ -126,6 +127,14 @@ void Summary::readPool( const ResPool & pool )
       {
         DBG << "<install>   ";
         to_be_installed[it->kind()].insert(it->resolvable());
+
+        if ( it->isKind( ResKind::package ) ) {
+          Package::constPtr package = asKind<Package>( it->resolvable() );
+          if ( package->isNeedreboot() ) {
+            _need_reboot_nonpatch = true;
+            _rebootNeeded[ResKind::package].insert( ResPair( nullptr, package ) );
+          }
+        }
       }
       if (it->status().isToBeUninstalled())
       {
@@ -1327,15 +1336,29 @@ void Summary::writeLocked( std::ostream & out )
 
 void Summary::writeRebootNeeded( std::ostream & out )
 {
-  if ( _rebootNeeded.empty() )
-    return;
+  auto generateLabel = [] ( zypp::ResKind kind, size_t count ) {
+    if ( kind == ResKind::patch )
+      return str::form(PL_("The following patch requires a system reboot:",
+                           "The following %d patches require a system reboot:",
+                          count ), static_cast<int>( count ) );
+    else if ( kind == ResKind::package )
+      return str::form(PL_("The following package requires a system reboot:",
+                           "The following %d packages require a system reboot:",
+                          count ), static_cast<int>( count ) );
+    return std::string();
+  };
 
-  const std::string & label( str::form(PL_("The following patch requires a system reboot:",
-					   "The following %d patches require a system reboot:",
-					   _rebootNeeded.size() ),
-				       (int)_rebootNeeded.size()) );
-  out << endl << ( ColorContext::MSG_WARNING << label ) << endl;
-  writeResolvableList( out, _rebootNeeded, ColorContext::MSG_WARNING );
+  auto checkAndPrintRebootNeeded = [ this, &generateLabel, &out ] ( zypp::ResKind kind ) {
+    const ResPairSet & resolvables = _rebootNeeded[kind];
+    if ( ! resolvables.empty() ) {
+      size_t count = resolvables.size();
+      out << endl << ( ColorContext::MSG_WARNING << generateLabel( kind, count ) ) << endl;
+      writeResolvableList( out, resolvables, ColorContext::MSG_WARNING );
+    }
+  };
+
+  checkAndPrintRebootNeeded( ResKind::patch );
+  checkAndPrintRebootNeeded( ResKind::package );
 }
 
 // --------------------------------------------------------------------------
@@ -1380,6 +1403,7 @@ void Summary::writePackageCounts( std::ostream & out )
     return;
 
   std::ostringstream s;
+  s << endl;
   bool gotcha = false;
   unsigned count;
   KindToResPairSet::const_iterator i;
@@ -1498,16 +1522,18 @@ void Summary::writePackageCounts( std::ostream & out )
 
 void Summary::dumpTo( std::ostream & out )
 {
+  // bsc#1180077: Take care nothing is written to stdout/err if there's
+  //              nothing todo. At least not if --quiet is used.
   Zypper & zypper( Zypper::instance() );
 
   struct SetColor
   {
     SetColor( bool force )
     : docolors( Zypper::instance().config().do_colors )
-    { if (force) Zypper::instance().config().do_colors = false; }
+    { if (force) Zypper::instance().configNoConst().do_colors = false; }
 
     ~SetColor()
-    { Zypper::instance().config().do_colors = docolors; }
+    { Zypper::instance().configNoConst().do_colors = docolors; }
 
     bool docolors;
   };
@@ -1525,6 +1551,13 @@ void Summary::dumpTo( std::ostream & out )
     unsetViewOption( SHOW_SUGGESTED );
   }
 
+  // bsc#993025: Suppress some information if verbosity is set to quiet
+  if ( zypper.config().verbosity == Out::QUIET )
+  {
+    unsetViewOption( SHOW_LOCKS );
+    unsetViewOption( SHOW_NOT_UPDATED );
+  }
+
   if ( _viewop & SHOW_LOCKS )
     writeLocked( out );
   if ( _viewop & SHOW_NOT_UPDATED )
@@ -1547,15 +1580,14 @@ void Summary::dumpTo( std::ostream & out )
     writeSupportNeedACC( out );
   }
   writeRebootNeeded( out );
-  out << endl;
   writePackageCounts( out );
   writeDownloadAndInstalledSizeSummary( out );
-  if ( _need_restart && zypper.runtimeData().plain_patch_command && !zypper.cOpts().count("updatestack-only") )
+  if ( _need_restart && zypper.runtimeData().plain_patch_command && !(_viewop & UPDATESTACK_ONLY) )
   {
     // patch command (auto)restricted to update stack patches
     Zypper::instance().out().notePar( 4, _("Package manager restart required. (Run this command once again after the update stack got updated)") );
   }
-  if ( _need_reboot )
+  if ( _need_reboot_patch || _need_reboot_nonpatch )
   {   Zypper::instance().out().notePar( 4, _("System reboot required.") ); }
 
   if ( !_ctc.empty() )
