@@ -88,6 +88,57 @@ enum TableLineStyle {
 
 class Table;
 
+///////////////////////////////////////////////////////////////////
+// Conditional Table column helpers.
+namespace ctcdetail
+{
+  /// Remember either \a _if or \a _else function.
+  template<class Tif_,class Telse_>
+  struct ColumnIf
+  {
+    ColumnIf( bool condition_r, std::function<Tif_()> if_r,  std::function<Telse_()> else_r )
+    { if ( condition_r ) _if = std::move(if_r); else _else = std::move(else_r); }
+    std::function<Tif_()> _if;
+    std::function<Telse_()> _else;
+  };
+  /// Specialization both functions return the same type
+  template<class Tif_>
+  struct ColumnIf<Tif_,Tif_>
+  {
+    ColumnIf( bool condition_r, std::function<Tif_()> if_r,  std::function<Tif_()> else_r )
+    : _ifelse { condition_r ? std::move(if_r) : std::move(else_r) }
+    {}
+    ColumnIf( bool condition_r, std::function<Tif_()> && if_r )
+    { if ( condition_r ) _ifelse = std::move(if_r); }
+    std::function<Tif_()> _ifelse;
+  };
+}
+///////////////////////////////////////////////////////////////////
+/// Conditional Table column factory.
+///
+/// Creates an conditional \ref TableRow or \ref TableHeader entry,
+/// depending on the value of \a condition_r. The columns content,
+/// if needed, is determined calling the function object matching the
+/// condition.
+///
+/// An empty function object will cause the column to be hidden and no
+/// content will be retrieved (see overloaded operator<<(TableRow &,..)).
+///
+/// \code
+///   TableRow()
+///     << "first"
+///     << ColumnIf( condition, [](){ return "second"; }, [](){ return "one but last"; } )
+///     << "last";
+/// \endcode
+template<class Tif_, class Telse_>
+auto ColumnIf( bool condition_r, Tif_ && if_r, Telse_ && else_r ) -> ctcdetail::ColumnIf<decltype(if_r()),decltype(else_r())>
+{ return { condition_r, std::forward<Tif_>(if_r), std::forward<Telse_>(else_r) }; }
+/** \overload no column on 'else' */
+template<class Tif_>
+auto ColumnIf( bool condition_r, Tif_ && if_r ) -> ctcdetail::ColumnIf<decltype(if_r()),decltype(if_r())>
+{ return { condition_r, std::forward<Tif_>(if_r) }; }
+
+
 class TableRow
 {
 private:
@@ -212,17 +263,25 @@ public:
   };
 
   const container & columns() const
-  { return _columns; }
+  { return _translateColumns ? _translatedColumns : _columns; }
 
   container & columns()
+  { return _translateColumns ? _translatedColumns : _columns; }
+
+  const container & columnsNoTr() const
   { return _columns; }
 
+  container & columnsNoTr()
+  { return _columns; }
+
+protected:
+  bool      _translateColumns = false;
 private:
   container _columns;
+  container _translatedColumns;
   container _details;
   ColorContext _ctxt;
   boost::any _userData;	///< user defined sort index, e.g. if string values don't work due to coloring
-  friend class Table;
 };
 
 /** \relates TableRow Add colummn. */
@@ -234,11 +293,31 @@ template<class Tp_>
 TableRow && operator<<( TableRow && tr, Tp_ && val )
 { return std::move( tr << std::forward<Tp_>(val) ); }
 
+/** \overload universal refernce for conditional Table columns */
+template<class Tif_, class Telse_> TableRow & operator<<( TableRow & tr, const ctcdetail::ColumnIf<Tif_,Telse_> & val )
+{ if ( val._if ) tr.add( val._if() ); else if ( val._else ) tr.add( val._else() ); return tr; }
+/** \overload universal refernce for conditional Table columns */
+template<class Tif_, class Telse_> TableRow & operator<<( TableRow & tr, ctcdetail::ColumnIf<Tif_,Telse_> & val )
+{ if ( val._if ) tr.add( val._if() ); else if ( val._else ) tr.add( val._else() ); return tr; }
+/** \overload universal refernce for conditional Table columns */
+template<class Tif_, class Telse_> TableRow & operator<<( TableRow & tr, ctcdetail::ColumnIf<Tif_,Telse_> && val )
+{ if ( val._if ) tr.add( val._if() ); else if ( val._else ) tr.add( val._else() ); return tr; }
+
+/** \overload universal refernce for conditional Table columns (specialized) */
+template<class Tif_> TableRow & operator<<( TableRow & tr, const ctcdetail::ColumnIf<Tif_,Tif_> & val )
+{ if ( val._ifelse ) tr.add( val._ifelse() ); return tr; }
+/** \overload universal refernce for conditional Table columns (specialized) */
+template<class Tif_> TableRow & operator<<( TableRow & tr, ctcdetail::ColumnIf<Tif_,Tif_> & val )
+{ if ( val._ifelse ) tr.add( val._ifelse() ); return tr; }
+/** \overload universal refernce for conditional Table columns (specialized) */
+template<class Tif_> TableRow & operator<<( TableRow & tr, ctcdetail::ColumnIf<Tif_,Tif_> && val )
+{ if ( val._ifelse ) tr.add( val._ifelse() ); return tr; }
+
 
 class TableHeader : public TableRow {
 public:
   //! Constructor. Reserve place for c columns.
-  TableHeader (unsigned c = 0): TableRow (c) {}
+  TableHeader (unsigned c = 0): TableRow (c) { _translateColumns = true; }
 };
 
 /** \relates TableHeader  Add column. */
@@ -288,7 +367,7 @@ public:
   void sort( std::list<unsigned> && byColumns_r )	{ if ( byColumns_r.size() ) _rows.sort( TableRow::Less( std::move(byColumns_r) ) ); }
 
   /** Custom sort */
-  template<class TCompare, typename std::enable_if<!std::is_integral<TCompare>::value>::type* = nullptr>
+  template<class TCompare, std::enable_if_t<!std::is_integral<TCompare>::value, int> = 0>
   void sort( TCompare && less_r )		{ _rows.sort( std::forward<TCompare>(less_r) ); }
 
   void lineStyle( TableLineStyle st );
