@@ -101,7 +101,7 @@ namespace zypp
     struct KeyRingReceive : public callback::ReceiveReport<KeyRingReport>
     {
       KeyRingReceive()
-          : _gopts(Zypper::instance().globalOpts())
+          : _gopts(Zypper::instance().config())
           {}
 
       ////////////////////////////////////////////////////////////////////
@@ -332,8 +332,6 @@ namespace zypp
           // translators: help text for the 'a' option in the 'r/t/a' prompt
           popts.setOptionHelp( (++off), _("Trust the key and import it into trusted keyring.") );
 
-        if (!zypper.globalOpts().non_interactive)
-          clear_keyboard_buffer();
         zypper.out().prompt(PROMPT_YN_GPG_KEY_TRUST, s.str(), popts);
         unsigned prep =
           get_prompt_reply(zypper, PROMPT_YN_GPG_KEY_TRUST, popts);
@@ -454,7 +452,7 @@ namespace zypp
       }
 
     private:
-      const GlobalOptions & _gopts;
+      const Config & _gopts;
     };
 
     ///////////////////////////////////////////////////////////////////
@@ -463,7 +461,7 @@ namespace zypp
 
     struct DigestReceive : public callback::ReceiveReport<DigestReport>
     {
-      DigestReceive() : _gopts(Zypper::instance().globalOpts()) {}
+      DigestReceive() : _gopts(Zypper::instance().config()) {}
 
       ////////////////////////////////////////////////////////////////////
 
@@ -527,8 +525,7 @@ namespace zypp
 	// translators: A prompt option help text
 	popts.setOptionHelp( 1, _("Discard the file.") );
 	popts.setShownCount( 1 );
-	if ( !zypper.globalOpts().non_interactive )
-	  clear_keyboard_buffer();
+
 	// translators: A prompt text
 	zypper.out().prompt( PROMPT_GPG_WRONG_DIGEST_ACCEPT, _("Unblock or discard?"), popts );
 	int reply = get_prompt_reply( zypper, PROMPT_GPG_WRONG_DIGEST_ACCEPT, popts );
@@ -536,7 +533,7 @@ namespace zypp
       }
 
     private:
-      const GlobalOptions & _gopts;
+      const Config & _gopts;
     };
 
    ///////////////////////////////////////////////////////////////////
