@@ -48,6 +48,18 @@ namespace text
   template <class Tltext, class Trtext>
   inline std::string join( const Tltext & ltext, const Trtext & rtext, const char * sep = " " )
   { std::string ret( asString(ltext) ); ret += sep; ret += asString(rtext); return ret; }
+
+  /** Whether the \a str_r ends with a WS. */
+  inline bool endsOnWS( const std::string & str_r )
+  {
+    bool ret = false;
+    if ( !str_r.empty() )
+    { int l = *str_r.rbegin(); if ( ::strchr( " \n\t", l ) ) ret = true; }
+    return ret;
+  }
+
+  inline const char * optBlankAfter( const std::string & str_r )
+  { return( endsOnWS( str_r ) ? "" : " " ); }
 }
 ///////////////////////////////////////////////////////////////////
 
@@ -211,7 +223,7 @@ namespace out
   {
     NON_COPYABLE( Writer );
 
-    Writer( std::ostream & str_r, ListLayout layout_r, const TFormater & formater_r )
+    Writer( std::ostream & str_r, const ListLayout & layout_r, const TFormater & formater_r )
     : _str( str_r )
     , _layout( layout_r )
     , _formater( formater_r )
@@ -691,7 +703,7 @@ public:
   virtual void warning(const std::string & msg, Verbosity verbosity = NORMAL, Type mask = TYPE_ALL) = 0;
 
   /** Convenience class for error reporting. */
-  class Error;
+  struct Error;
 
   /**
    * Show an error message and an optional hint.
@@ -724,7 +736,7 @@ public:
   class ProgressBar;
 
   /** Convenience class for download progress output. */
-  class DownloadProgress;
+  struct DownloadProgress;
 
   /**
    * Start of an operation with reported progress.
@@ -1005,7 +1017,7 @@ public:
     _progress.noSend();	// suppress ~ProgressData final report
     if ( indeterminate( _error ) )
       _error = ( _progress.reportValue() != 100 && _progress.reportPercent() );
-    _out.progressEnd( _progressId, outLabel( _progress.name() ), _error );
+    _out.progressEnd( _progressId, outLabel( _progress.name() ), bool(_error) );
   }
 
   /** Immediately print the progress bar not waiting for a new trigger. */
