@@ -10,6 +10,7 @@
 
 #include <zypp/ZYppFactory.h>
 #include <zypp/base/Logger.h>
+#include <zypp/TriBool.h>
 #include <zypp/FileChecker.h>
 #include <zypp/base/InputStream.h>
 #include <zypp/base/IOStream.h>
@@ -22,9 +23,10 @@
 #include "utils/misc.h"
 #include "utils/prompt.h"	// Continue? and solver problem prompt
 #include "utils/pager.h"	// to view the summary
-#include "Summary.h"
+#include "global-settings.h"
 
 #include "solve-commit.h"
+#include "commands/needs-rebooting.h"
 
 using namespace zypp;
 extern ZYpp::Ptr God;
@@ -42,17 +44,24 @@ static TriBool show_problem( Zypper & zypper, const ResolverProblem & prob, Prob
   if ( !tmp.empty() )
     desc_stm << "  " << tmp << endl;
 
+  std::ostringstream desc_ext_stm;
+  desc_ext_stm << _("Detailed information: ") << endl;
+  for ( const std::string &pRule : prob.completeProblemInfo() ) {
+    desc_ext_stm << "\t" << pRule << endl;
+  }
+
   const ProblemSolutionList & solutions = prob.solutions();
 
   unsigned n = 0;
+  std::ostringstream solution_stm;
   for ( const auto & solPtr : solutions )
   {
     ++n;
     // TranslatorExplanation %d is the solution number
-    desc_stm << str::Format(_(" Solution %d: ")) % n << solPtr->description() << endl;
+    solution_stm << str::Format(_(" Solution %d: ")) % n << solPtr->description() << endl;
     tmp = solPtr->details ();
     if ( !tmp.empty () )
-      desc_stm << indent( tmp, 2 ) << endl;
+      solution_stm << indent( tmp, 2 ) << endl;
   }
 
   unsigned problem_count = God->resolver()->problems().size();
@@ -61,7 +70,7 @@ static TriBool show_problem( Zypper & zypper, const ResolverProblem & prob, Prob
   // without solutions, its useless to prompt
   if ( solutions.empty() )
   {
-    zypper.out().error( desc_stm.str() );
+    zypper.out().error( desc_stm.str() + solution_stm.str() );
     return false;
   }
 
@@ -85,6 +94,11 @@ static TriBool show_problem( Zypper & zypper, const ResolverProblem & prob, Prob
   for ( unsigned i = 1; i <= solution_count; i++ )
     numbers << i << "/";
 
+  unsigned extInfoAnswer = 0;
+  unsigned cancelAnswer = 0;
+  int retryAnswer = -1;
+  int skipAnswer = -1;
+
   if ( problem_count > 1 )
   {
     default_reply = solution_count + 2;
@@ -92,9 +106,13 @@ static TriBool show_problem( Zypper & zypper, const ResolverProblem & prob, Prob
     // "Choose from above solutions by number or skip, retry or cancel"
     // Translate the letters to whatever is suitable for your language.
     // The anserws must be separated by slash characters '/' and must
-    // correspond to skip/retry/cancel in that order.
+    // correspond to skip/retry/cancel/detailed information in that order.
     // The answers should be lower case letters.
-    popts.setOptions( numbers.str() + _("s/r/c"), default_reply );
+    popts.setOptions( numbers.str() + _("s/r/c/d"), default_reply );
+    skipAnswer    = solution_count;
+    retryAnswer   = solution_count + 1;
+    cancelAnswer  = solution_count + 2;
+    extInfoAnswer = solution_count + 3;
   }
   else
   {
@@ -102,30 +120,56 @@ static TriBool show_problem( Zypper & zypper, const ResolverProblem & prob, Prob
     // translators: answers for dependency problem solution input prompt:
     // "Choose from above solutions by number or cancel"
     // Translate the letter 'c' to whatever is suitable for your language
-    // and to the same as you translated it in the "s/r/c" string
-    // See the "s/r/c" comment for other details.
+    // and to the same as you translated it in the "s/r/c/d" string
+    // See the "s/r/c/d" comment for other details.
     // One letter string  for translation can be tricky, so in case of problems,
-    // please report a bug against zypper at bugzilla.novell.com, we'll try to solve it.
-    popts.setOptions( numbers.str() + _("c"), default_reply );
+    // please report a bug against zypper at bugzilla.suse.com, we'll try to solve it.
+    popts.setOptions( numbers.str() + _("c/d"), default_reply );
+    cancelAnswer  = solution_count;
+    extInfoAnswer = solution_count + 1;
   }
 
-  if ( !zypper.globalOpts().non_interactive )
-    clear_keyboard_buffer();
-  zypper.out().prompt( PROMPT_DEP_RESOLVE, prompt_text, popts, desc_stm.str() );
-  unsigned reply = get_prompt_reply( zypper, PROMPT_DEP_RESOLVE, popts );
+  for ( unsigned i = 0; i < solution_count; i++ )
+    popts.setOptionHelp( i, str::Format( _("Choose solution %1%") ) % (i+1) );
+  if ( skipAnswer != -1 ) {
+    popts.setOptionHelp( skipAnswer,  _("Skip problem and continue.") );
+  }
+  if ( retryAnswer != -1 ) {
+    popts.setOptionHelp( retryAnswer, _("Retry solving immediately.") );
+  }
+  popts.setOptionHelp( cancelAnswer,  _("Choose no solution and cancel.") );
+  popts.setOptionHelp( extInfoAnswer, _("Toggle show detailed conflict information.") );
 
-  // retry
-  if ( problem_count > 1 && reply == solution_count + 1 )
-    return true;
-  // cancel (one problem)
-  if ( problem_count == 1 && reply == solution_count )
-    return false;
-  // cancel (more problems)
-  if ( problem_count > 1 && reply == solution_count + 2 )
-    return false;
-  // skip
-  if ( problem_count > 1 && reply == solution_count )
-    return indeterminate; // continue with next problem
+  bool showExtInfo = false;
+  unsigned reply = default_reply;
+
+  while ( true ) {
+
+    std::ostringstream fulltext;
+    fulltext << desc_stm.str();
+    if ( showExtInfo )
+      fulltext << desc_ext_stm.str();
+    fulltext << solution_stm.str();
+
+    zypper.out().prompt( PROMPT_DEP_RESOLVE, prompt_text, popts, fulltext.str() );
+    reply = get_prompt_reply( zypper, PROMPT_DEP_RESOLVE, popts );
+
+    if ( reply == extInfoAnswer ) {
+      showExtInfo = !showExtInfo;
+      continue;
+    }
+    // cancel
+    if ( reply == cancelAnswer )
+      return false;
+    // retry
+    if (  reply == static_cast<unsigned>(retryAnswer) )
+      return true;
+    // skip
+    if ( reply == static_cast<unsigned>(skipAnswer) )
+      return indeterminate; // continue with next problem
+
+    break;
+  }
 
   zypper.out().info( str::Format(_("Applying solution %s")) % (reply + 1), Out::HIGH );
   ProblemSolutionList::const_iterator reply_i = solutions.begin();
@@ -169,7 +213,7 @@ static bool show_problems( Zypper & zypper )
     TriBool stopnow = show_problem( zypper, *probPtr, todo );
     if ( !indeterminate( stopnow ) )
     {
-      retry = stopnow;
+      retry = bool(stopnow);
       break;
     }
   }
@@ -210,16 +254,8 @@ static void set_force_resolution( Zypper & zypper )
   // --force-resolution command line parameter value
   TriBool force_resolution = zypper.runtimeData().force_resolution;
 
-  if ( zypper.cOpts().count("force-resolution") )
-    force_resolution = true;
-  if ( zypper.cOpts().count("no-force-resolution") )
-  {
-    if ( force_resolution )
-      zypper.out().warning(str::form(
-        // translators: meaning --force-resolution and --no-force-resolution
-        _("%s conflicts with %s, will use the less aggressive %s"),
-          "--force-resolution", "--no-force-resolution", "--no-force-resolution"));
-    force_resolution = false;
+  if ( !indeterminate( SolverSettings::instance()._forceResolution ) ) {
+    force_resolution = SolverSettings::instance()._forceResolution;
   }
 
   // if --force-resolution was not specified on the command line,
@@ -234,7 +270,7 @@ static void set_force_resolution( Zypper & zypper )
   // bnc #369980
   if ( indeterminate(force_resolution) )
   {
-    if ( !zypper.globalOpts().non_interactive && zypper.command() == ZypperCommand::REMOVE )
+    if ( !zypper.config().non_interactive && zypper.command() == ZypperCommand::REMOVE )
       force_resolution = true;
     else
       force_resolution = false;
@@ -245,19 +281,20 @@ static void set_force_resolution( Zypper & zypper )
 
   DBG << "force resolution: " << force_resolution << endl;
   std::ostringstream s;
-  s << _("Force resolution:") << " " << asYesNo( force_resolution );
+  s << _("Force resolution:") << " " << asYesNo( bool(force_resolution) );
   zypper.out().info(s.str(), Out::HIGH);
 
-  God->resolver()->setForceResolve( force_resolution );
+  God->resolver()->setForceResolve( bool(force_resolution) );
 }
 
 static void set_clean_deps( Zypper & zypper )
 {
   if ( zypper.command() == ZypperCommand::REMOVE )
   {
-    if ( zypper.cOpts().find("clean-deps") != zypper.cOpts().end() )
+    const auto &solverSettings = SolverSettings::instance();
+    if ( solverSettings._cleanDeps == true )
       God->resolver()->setCleandepsOnRemove(true);
-    else if ( zypper.cOpts().find("no-clean-deps") != zypper.cOpts().end() )
+    else if ( solverSettings._cleanDeps == false )
       God->resolver()->setCleandepsOnRemove(false);
   }
   DBG << "clean deps on remove: " << God->resolver()->cleandepsOnRemove() << endl;
@@ -267,14 +304,16 @@ static void set_no_recommends( Zypper & zypper )
 {
   bool no_recommends = !zypper.config().solver_installRecommends;
 
+  const auto &solverSettings = SolverSettings::instance();
+
   // override zypper.conf in these cases:
   if ( zypper.command() == ZypperCommand::REMOVE )
     // never install recommends when removing packages
     no_recommends = true;
-  else if ( zypper.cOpts().count("no-recommends") )
+  else if ( solverSettings._recommends == false )
     // install also recommended packages unless --no-recommends is specified
     no_recommends = true;
-  else if ( zypper.cOpts().count("recommends") )
+  else if ( solverSettings._recommends == true )
     no_recommends = false;
 
   DBG << "no recommends (only requires): " << no_recommends << endl;
@@ -285,7 +324,7 @@ static void set_no_recommends( Zypper & zypper )
 static void set_ignore_recommends_of_installed(Zypper & zypper)
 {
   bool ignore = true;
-  if ( ( zypper.command() == ZypperCommand::DIST_UPGRADE && !zypper.cOpts().count("no-recommends") )
+  if ( ( zypper.command() == ZypperCommand::DIST_UPGRADE && SolverSettings::instance()._recommends != false )
     || zypper.command() == ZypperCommand::INSTALL_NEW_RECOMMENDS )
     ignore = false;
   DBG << "ignore recommends of already installed packages: " << ignore << endl;
@@ -304,23 +343,22 @@ static void set_solver_flags( Zypper & zypper )
   // solve_with_update must be false until the command passed the initialization!
   God->resolver()->setUpdateMode( zypper.runtimeData().solve_with_update );
 
+  const auto &solverSettings = SolverSettings::instance();
+  God->resolver()->setFocus( solverSettings._focus );
   // Use resolver->dupSet... if ZypperCommand::DIST_UPGRADE
-  if ( shared_ptr<InstallerBaseOptions> installOptions = zypper.commandOptionsAs<InstallerBaseOptions>() )
+  if ( zypper.command() == ZypperCommand::DIST_UPGRADE )
   {
-    if ( zypper.command() == ZypperCommand::DIST_UPGRADE )
-    {
-      if ( installOptions->_allowDowngrade != -1 ) God->resolver()->dupSetAllowDowngrade( installOptions->_allowDowngrade );
-      if ( installOptions->_allowNameChange != -1 ) God->resolver()->dupSetAllowNameChange( installOptions->_allowNameChange );
-      if ( installOptions->_allowArchChange != -1 ) God->resolver()->dupSetAllowArchChange( installOptions->_allowArchChange );
-      if ( installOptions->_allowVendorChange != -1 ) God->resolver()->dupSetAllowVendorChange( installOptions->_allowVendorChange );
-    }
-    else
-    {
-      if ( installOptions->_allowDowngrade != -1 ) God->resolver()->setAllowDowngrade( installOptions->_allowDowngrade );
-      if ( installOptions->_allowNameChange != -1 ) God->resolver()->setAllowNameChange( installOptions->_allowNameChange );
-      if ( installOptions->_allowArchChange != -1 ) God->resolver()->setAllowArchChange( installOptions->_allowArchChange );
-      if ( installOptions->_allowVendorChange != -1 ) God->resolver()->setAllowVendorChange( installOptions->_allowVendorChange );
-    }
+    if ( !indeterminate( solverSettings._allowDowngrade ) ) God->resolver()->dupSetAllowDowngrade( bool(solverSettings._allowDowngrade) );
+    if ( !indeterminate( solverSettings._allowNameChange ) ) God->resolver()->dupSetAllowNameChange( bool(solverSettings._allowNameChange) );
+    if ( !indeterminate( solverSettings._allowArchChange ) ) God->resolver()->dupSetAllowArchChange( bool(solverSettings._allowArchChange) );
+    if ( !indeterminate( solverSettings._allowVendorChange ) ) God->resolver()->dupSetAllowVendorChange( bool(solverSettings._allowVendorChange) );
+  }
+  else
+  {
+    if ( !indeterminate( solverSettings._allowDowngrade ) ) God->resolver()->setAllowDowngrade( bool(solverSettings._allowDowngrade) );
+    if ( !indeterminate( solverSettings._allowNameChange ) ) God->resolver()->setAllowNameChange( bool(solverSettings._allowNameChange) );
+    if ( !indeterminate( solverSettings._allowArchChange ) ) God->resolver()->setAllowArchChange( bool(solverSettings._allowArchChange) );
+    if ( !indeterminate( solverSettings._allowVendorChange ) ) God->resolver()->setAllowVendorChange( bool(solverSettings._allowVendorChange) );
   }
 }
 
@@ -356,36 +394,38 @@ static bool dist_upgrade( Zypper & zypper )
   // If those are specified addUpgradeRepo and solve,
   // otherwise perform a full dist upgrade.
 
-  std::list<RepoInfo> specified;
-  std::list<std::string> not_found;
-  parsed_opts::const_iterator tmp1;
-  if ( (tmp1 = copts.find("from")) != copts.end() )
-    get_repos( zypper, tmp1->second.begin(), tmp1->second.end(), specified, not_found );
-  report_unknown_repos( zypper.out(), not_found );
+  auto & dupSettings = DupSettings::instance();
+  if ( dupSettings._fromRepos.size() ) {
+    std::list<RepoInfo> specified;
+    std::list<std::string> not_found;
 
-  if ( !not_found.empty() )
-  {
-    zypper.setExitCode( ZYPPER_EXIT_ERR_INVALID_ARGS );
-    ZYPP_THROW( ExitRequestException("Some of specified repositories were not found.") );
-  }
+    get_repos( zypper, dupSettings._fromRepos.begin(), dupSettings._fromRepos.end(), specified, not_found );
+    report_unknown_repos( zypper.out(), not_found );
 
-  if ( ! specified.empty() )
-  {
-    // Here: do upgrade for the specified repos:
-    Resolver_Ptr resolver( God->resolver() );
-    ResPool      pool    ( God->pool() );
-    for_( it, specified.begin(), specified.end() )
+    if ( !not_found.empty() )
     {
-      Repository repo( pool.reposFind( it->alias() ) );
-      MIL << "Adding upgrade repository: " << repo.alias() << endl;
-      resolver->addUpgradeRepo( repo );
+      zypper.setExitCode( ZYPPER_EXIT_ERR_INVALID_ARGS );
+      ZYPP_THROW( ExitRequestException("Some of specified repositories were not found.") );
     }
 
-    DBG << "Calling the solver..." << endl;
-    //! \todo Somehow tell set_solver_flags/set_ignore_recommends_of_installed that
-    //! this is no full upgrade. Until then set setIgnoreAlreadyRecommended again here:
-    resolver->setIgnoreAlreadyRecommended( true );
-    return resolver->resolvePool();
+    if ( ! specified.empty() )
+    {
+      // Here: do upgrade for the specified repos:
+      Resolver_Ptr resolver( God->resolver() );
+      ResPool      pool    ( God->pool() );
+      for_( it, specified.begin(), specified.end() )
+      {
+        Repository repo( pool.reposFind( it->alias() ) );
+        MIL << "Adding upgrade repository: " << repo.alias() << endl;
+        resolver->addUpgradeRepo( repo );
+      }
+
+      DBG << "Calling the solver..." << endl;
+      //! \todo Somehow tell set_solver_flags/set_ignore_recommends_of_installed that
+      //! this is no full upgrade. Until then set setIgnoreAlreadyRecommended again here:
+      resolver->setIgnoreAlreadyRecommended( true );
+      return resolver->resolvePool();
+    }
   }
 
   // Here: compute the full upgrade
@@ -414,20 +454,15 @@ static void make_solver_test_case( Zypper & zypper )
   }
 }
 
-ZYppCommitPolicy get_commit_policy( Zypper & zypper )
+ZYppCommitPolicy get_commit_policy( Zypper & zypper, DownloadMode dlMode_r )
 {
   ZYppCommitPolicy policy;
-  const parsed_opts & opts = zypper.cOpts();
 
-  if ( opts.find("dry-run") != opts.end() )
+  if ( DryRunSettings::instance().isEnabled() )
     policy.dryRun(true);
 
-  if ( opts.find("download") != opts.end()
-    || opts.find("download-only") != opts.end()
-    || opts.find("download-in-advance") != opts.end()
-    || opts.find("download-in-heaps") != opts.end()
-    || opts.find("download-as-needed") != opts.end() )
-    policy.downloadMode(get_download_option(zypper));
+  if ( dlMode_r != DownloadDefault )
+    policy.downloadMode( dlMode_r );
 
   policy.syncPoolAfterCommit( policy.dryRun() ? false : zypper.runningShell() );
 
@@ -441,38 +476,37 @@ ZYppCommitPolicy get_commit_policy( Zypper & zypper )
  */
 static void notify_processes_using_deleted_files( Zypper & zypper )
 {
-  if ( ! zypper.config().psCheckAccessDeleted )
-  {
+  if ( ! zypper.config().psCheckAccessDeleted ) {
     zypper.out().info( str::form(_("Check for running processes using deleted libraries is disabled in zypper.conf. Run '%s' to check manually."),
 				 "zypper ps -s" ) );
-    return;	// disabled in config
-  }
-
-  zypper.out().info(_("Checking for running processes using deleted libraries..."), Out::HIGH );
-  CheckAccessDeleted checker( false ); // wait for explicit call to check()
-  try
-  {
-    checker.check();
-  }
-  catch( const Exception & e )
-  {
-    if ( zypper.out().verbosity() > Out::NORMAL )
+  } else {
+    zypper.out().info(_("Checking for running processes using deleted libraries..."), Out::HIGH );
+    CheckAccessDeleted checker( false ); // wait for explicit call to check()
+    try
     {
-      if ( e.historySize() )
-	zypper.out().error( e, _("Check failed:") );
-      else
-	zypper.out().info( str::Str() << ( ColorContext::MSG_WARNING << _("Skip check:") ) << " " << e.asUserString() );
+      checker.check();
+    }
+    catch( const Exception & e )
+    {
+      if ( zypper.out().verbosity() > Out::NORMAL )
+      {
+        if ( e.historySize() )
+    zypper.out().error( e, _("Check failed:") );
+        else
+    zypper.out().info( str::Str() << ( ColorContext::MSG_WARNING << _("Skip check:") ) << " " << e.asUserString() );
+      }
+    }
+
+    // Don't suggest "zypper ps" if zypper is the only prog with deleted open files.
+    if ( checker.size() > 1 || ( checker.size() == 1 && checker.begin()->pid != str::numstring(::getpid()) ) )
+    {
+      zypper.out().info( str::Format(_("There are running programs which still use files and libraries deleted or updated by recent upgrades. They should be restarted to benefit from the latest updates. Run '%1%' to list these programs.") )
+      % "zypper ps -s" );
     }
   }
 
-  // Don't suggest "zypper ps" if zypper is the only prog with deleted open files.
-  if ( checker.size() > 1 || ( checker.size() == 1 && checker.begin()->pid != str::numstring(::getpid()) ) )
-  {
-    zypper.out().info(str::form(
-        _("There are some running programs that might use files deleted by recent upgrade."
-          " You may wish to check and restart some of them. Run '%s' to list these programs."),
-        "zypper ps -s"));
-  }
+  zypper.out().info(" ");
+  NeedsRebootingCmd::checkRebootNeeded( zypper, indeterminate );
 }
 
 static void show_update_messages( Zypper & zypper, const UpdateNotifications & messages )
@@ -486,11 +520,11 @@ static void show_update_messages( Zypper & zypper, const UpdateNotifications & m
   std::ostringstream msg;
   for_( it, messages.begin(), messages.end() )
   {
-    MIL << "- From " << it->solvable().asString() << " in file " << Pathname::showRootIf( zypper.globalOpts().root_dir, it->file() ) << endl;
-    zypper.out().info( it->solvable().asString() + " (" + Pathname::showRootIf(zypper.globalOpts().root_dir, it->file()) + ")" );
+    MIL << "- From " << it->solvable().asString() << " in file " << Pathname::showRootIf( zypper.config().root_dir, it->file() ) << endl;
+    zypper.out().info( it->solvable().asString() + " (" + Pathname::showRootIf(zypper.config().root_dir, it->file()) + ")" );
     {
       msg << str::form(_("Message from package %s:"), it->solvable().name().c_str() ) << endl << endl;
-      InputStream istr( Pathname::assertprefix( zypper.globalOpts().root_dir, it->file() ) );
+      InputStream istr( Pathname::assertprefix( zypper.config().root_dir, it->file() ) );
       iostr::copy( istr, msg );
       msg << endl << "-----------------------------------------------------------------------------" << endl;
     }
@@ -501,8 +535,6 @@ static void show_update_messages( Zypper & zypper, const UpdateNotifications & m
   std::string prompt_text( _("View the notifications now?") );
   unsigned reply;
 
-  if ( !zypper.globalOpts().non_interactive )
-    clear_keyboard_buffer();
   zypper.out().prompt( PROMPT_YN_INST_REMOVE_CONTINUE, prompt_text, popts );
   reply = get_prompt_reply( zypper, PROMPT_YN_INST_REMOVE_CONTINUE, popts );
 
@@ -524,10 +556,11 @@ static void show_update_messages( Zypper & zypper, const UpdateNotifications & m
  *  ZYPPER_EXIT_INF_REBOOT_NEEDED - if one of patches to be installed needs machine reboot,
  *  ZYPPER_EXIT_INF_RESTART_NEEDED - if one of patches to be installed needs package manager restart
  */
-void solve_and_commit ( Zypper & zypper )
+
+void solve_and_commit (Zypper & zypper , Summary::ViewOptions summaryOptions_r, DownloadMode dlMode_r , SolveAndCommitPolicy commitPolicy_r )
 {
   bool need_another_solver_run = true;
-  bool dryRunEtc = zypper.cOpts().count("dry-run") || ( get_download_option( zypper, true ) == DownloadOnly );
+  bool dryRunEtc = DryRunSettings::instance().isEnabled() || ( dlMode_r == DownloadOnly );
   do
   {
     // CALL SOLVER
@@ -554,7 +587,7 @@ void solve_and_commit ( Zypper & zypper )
         }
 
         // go on, we've got solution or we don't want a solution (we want testcase)
-        if ( success || zypper.cOpts().count("debug-solver") )
+        if ( success || SolverSettings::instance()._debugSolver )
           break;
 
         success = show_problems( zypper );
@@ -566,7 +599,7 @@ void solve_and_commit ( Zypper & zypper )
       }
     }
 
-    if ( zypper.cOpts().count("debug-solver") )
+    if ( SolverSettings::instance()._debugSolver )
     {
       make_solver_test_case( zypper );
       return;	// ZYPPER_EXIT_OK
@@ -576,42 +609,20 @@ void solve_and_commit ( Zypper & zypper )
 
     // SHOW SUMMARY
 
-    Summary summary( God->pool() );
+    Summary summary( God->pool(), summaryOptions_r );
 
-    if ( zypper.cOpts().count("details") )
-      summary.setViewOption( Summary::DETAILS );
-    else if ( zypper.out().verbosity() == Out::HIGH )
+    if ( zypper.out().verbosity() == Out::HIGH )
       summary.setViewOption( Summary::SHOW_VERSION );
     else if ( zypper.out().verbosity() == Out::DEBUG )
       summary.setViewOption( Summary::SHOW_ALL );
 
-    // show not updated packages if 'zypper up' (-t package or -t product)
-    ResKindSet kinds;
-    if ( zypper.cOpts().find("type") != zypper.cOpts().end() )
-      kinds = kindset_from( zypper.cOpts().find("type")->second );
-    if ( zypper.command() == ZypperCommand::UPDATE
-      && zypper.arguments().empty()
-      && ( kinds.empty()
-        || kinds.find(ResKind::package) != kinds.end()
-	|| kinds.find(ResKind::product) != kinds.end() ) )
-    {
-      summary.setViewOption( Summary::SHOW_NOT_UPDATED );
-    }
-
-    if ( zypper.command() == ZypperCommand::PATCH
-      || zypper.command() == ZypperCommand::UPDATE
-      || zypper.command() == ZypperCommand::DIST_UPGRADE )
-    {
-      summary.setViewOption( Summary::SHOW_LOCKS );
-    }
-
     // if running on SUSE Linux Enterprise, report unsupported packages
     if ( runningOnEnterprise() )
       summary.setViewOption( Summary::SHOW_UNSUPPORTED );
     else
       summary.unsetViewOption( Summary::SHOW_UNSUPPORTED );
 
-    if ( get_download_option( zypper, true ) == DownloadOnly )
+    if ( dlMode_r == DownloadOnly )
       summary.setDownloadOnly( true );
 
     // show the summary
@@ -631,7 +642,7 @@ void solve_and_commit ( Zypper & zypper )
 
       // check root user
       if ( zypper.command() == ZypperCommand::VERIFY && geteuid() != 0
-	&& !zypper.globalOpts().changedRoot )
+	&& !zypper.config().changedRoot )
       {
         zypper.out().error(_("Root privileges are required to fix broken package dependencies.") );
         zypper.setExitCode( ZYPPER_EXIT_ERR_PRIVILEGES );
@@ -686,8 +697,6 @@ void solve_and_commit ( Zypper & zypper )
       unsigned reply;
       do
       {
-        if ( !zypper.globalOpts().non_interactive )
-          clear_keyboard_buffer();
         zypper.out().prompt( PROMPT_YN_INST_REMOVE_CONTINUE, prompt_text, popts );
         reply = get_prompt_reply( zypper, PROMPT_YN_INST_REMOVE_CONTINUE, popts );
 
@@ -787,12 +796,12 @@ void solve_and_commit ( Zypper & zypper )
 	  {
 	    std::ostringstream s;
 	    s << _("committing");
-	    if ( copts.count("dry-run") )
+	    if ( DryRunSettings::instance().isEnabled() )
 	      s << " " << _("(dry run)");
 	    zypper.out().info( s.str(), Out::HIGH );
 	  }
 
-          ZYppCommitResult result = God->commit( get_commit_policy( zypper ) );
+          ZYppCommitResult result = God->commit( get_commit_policy( zypper, dlMode_r ) );
           gData.show_media_progress_hack = false;
 	  gData.entered_commit = false;
 
@@ -825,7 +834,7 @@ void solve_and_commit ( Zypper & zypper )
 	  {
 	    try
 	    {
-	      RepoManager manager( zypper.globalOpts().rm_options );
+	      RepoManager manager( zypper.config().rm_options );
 	      for( RepoInfo::urls_const_iterator it = e.info().baseUrlsBegin(); it != e.info().baseUrlsEnd(); ++it )
 	      {
 		if ( manager.checkIfToRefreshMetadata( e.info(), *it, RepoManager::RefreshForced ) == RepoManager::REFRESH_NEEDED )
@@ -885,7 +894,7 @@ void solve_and_commit ( Zypper & zypper )
 	  //! \todo This won't be necessary once we get a new solver flag
 	  //! for installing source packages without their build deps
 	  if ( !zypper.runtimeData().srcpkgs_to_install.empty() )
-	    install_src_pkgs( zypper );
+	    install_src_pkgs( zypper, dlMode_r );
 
 	  // set return value to 'reboot needed'
 	  if ( summary.needMachineReboot() )
@@ -906,7 +915,7 @@ void solve_and_commit ( Zypper & zypper )
 	}
 
         // check for running services (fate #300763)
-        if ( !( zypper.globalOpts().changedRoot || dryRunEtc )
+        if ( !( zypper.config().changedRoot || dryRunEtc )
 	  && ( summary.packagesToRemove() || summary.packagesToUpgrade() || summary.packagesToDowngrade() ) )
 	{
           notify_processes_using_deleted_files( zypper );
@@ -916,6 +925,10 @@ void solve_and_commit ( Zypper & zypper )
     // noting to do
     else
     {
+      //used to make sure locales and metadata is written even if there is no packages to be installed/removed
+      if ( commitPolicy_r == ForceCommit )
+        God->commit( ZYppCommitPolicy() );
+
       if ( zypper.command() == ZypperCommand::VERIFY )
         zypper.out().info(_("Dependencies of all installed packages are satisfied.") );
       else
