@@ -13,82 +13,27 @@
 #include "Table.h"
 #include "update.h"
 #include "main.h"
+#include "global-settings.h"
+#include "utils/misc.h"
 
 using namespace zypp;
 typedef std::set<PoolItem> Candidates;
 
 extern ZYpp::Ptr God;
 
-static void find_updates( const ResKindSet & kinds, Candidates & candidates );
+static void find_updates( const ResKindSet & kinds, Candidates & candidates, bool all_r );
 
 ///////////////////////////////////////////////////////////////////
-/// \class Issues
-/// \brief An issue (Type,Id) pair
-///////////////////////////////////////////////////////////////////
-struct Issue : std::pair<std::string, std::string>
+/// will go into next libzypp
+namespace zypp_pending
 {
-  Issue( std::string issueType_r, std::string issueId_r )
-  : std::pair<std::string, std::string>( std::move(issueType_r), std::move(issueId_r) )
-  {}
-
-  std::string &       type()			{ return first; }
-  const std::string & type()		const	{ return first; }
-  bool                anyType()		const	{ return type().empty(); }
-  bool                specificType()	const	{ return !anyType(); }
-
-  std::string &       id()			{ return second; }
-  const std::string & id()		const	{ return second; }
-  bool                anyId()		const 	{ return id().empty(); }
-  bool                specificId()	const	{ return !anyId(); }
-};
-
-///////////////////////////////////////////////////////////////////
-/// \class CliScanIssues
-/// \brief Setup issue (Type,Id) pairs from CLI
-///////////////////////////////////////////////////////////////////
-struct CliScanIssues : public std::set<Issue>
-{
-  CliScanIssues()
+  template<class TMap>
+  Iterable<typename MapKVIteratorTraits<TMap>::Key_const_iterator> make_map_key_Iterable( const TMap & map_r )
   {
-    //        cliOption   issueType
-    checkCLI( "issues",   ""/*any*/ );
-    checkCLI( "bugzilla", "bugzilla" );
-    checkCLI( "bz",       "bugzilla" );
-    checkCLI( "cve",      "cve" );
+    return makeIterable( make_map_key_begin( map_r ), make_map_key_end( map_r ) );
   }
-
-private:
-  void checkCLI( const std::string & cliOption_r, const std::string & issueType_r )
-  {
-    Zypper & zypper( Zypper::instance() );
-
-    bool anyId = false;	// plain option without opt. args
-    std::vector<std::string> issueIds;
-
-    for ( const auto & val : zypper.cOptValues( cliOption_r ) )
-    {
-      if ( str::split( val, std::back_inserter(issueIds), "," ) == 0 )
-      {	anyId = true; }
-    }
-
-    if ( issueIds.empty() )
-    {
-      if ( anyId )
-      { insert( value_type( issueType_r, std::string() ) ); }
-    }
-    else
-    {
-      if ( anyId )
-      {
-	zypper.out().warning(str::form(
-	  _("Ignoring %s without argument because similar option with an argument has been specified."),
-	  ("--" + cliOption_r).c_str() ));
-      }
-      for ( auto & val : issueIds )
-      { insert( value_type( issueType_r, std::move(val) ) ); }
-    }
-  }
-};
+} // namespce zypp
+///////////////////////////////////////////////////////////////////
 
 ///////////////////////////////////////////////////////////////////
 namespace
@@ -100,10 +45,121 @@ namespace
   {
     return pi.isBroken()
     && ! pi.isUnwanted()
-    && ! ( Zypper::instance().globalOpts().exclude_optional_patches && pi->asKind<Patch>()->categoryEnum() == Patch::CAT_OPTIONAL )
+    && ! ( Zypper::instance().config().exclude_optional_patches && pi->asKind<Patch>()->categoryEnum() == Patch::CAT_OPTIONAL )
     && pi->asKind<Patch>()->restartSuggested();
   }
 
+  /** RNC: Print other-update element */
+  inline std::ostream & xmlPrintOtherUpdateOn( std::ostream & str, const PoolItem & pi_r )
+  {
+    xmlout::Node parent { cout, "update", xmlout::Node::optionalContent, {
+      { "kind", pi_r.kind() },
+      { "name", pi_r.name () },
+      { "edition", pi_r.edition() },
+      { "arch", pi_r.arch() },
+    } };
+    // for packages show also the current installed version (bnc #466599)
+    {
+      const PoolItem & ipi( ui::Selectable::get(pi_r)->installedObj() );
+      if ( ipi )
+      {
+	if ( pi_r.edition() != ipi.edition() )
+	  parent.addAttr( { "edition-old", ipi.edition() } );
+	if ( pi_r.arch() != ipi.arch() )
+	  parent.addAttr( { "arch-old", ipi.arch() } );
+      }
+    }
+
+    dumpAsXmlOn( *parent, pi_r.summary(), "summary" );
+    dumpAsXmlOn( *parent, pi_r.description(), "description" );
+    dumpAsXmlOn( *parent, pi_r.licenseToConfirm(), "license" );
+
+    if ( !pi_r.repoInfo().alias().empty() )
+    {
+      xmlout::Node( *parent, "source", xmlout::Node::optionalContent, {
+	{ "url", pi_r.repoInfo().url().asString() },
+	{ "alias", pi_r.repoInfo().alias() },
+      } );
+    }
+    return str;
+  }
+
+  /** RNC: Print patch-issue-list-element */
+  inline std::ostream & xmlPrintPatchIssueListElementOn( std::ostream & str, Patch::ReferenceIterator begin_r, Patch::ReferenceIterator end_r )
+  {
+    xmlout::Node parent { str, "issue-list", xmlout::Node::optionalContent };
+    for_( it, begin_r, end_r )
+    {
+      xmlout::Node issue { *parent, "issue", xmlout::Node::optionalContent, {
+	{ "type", it.type() },
+	{ "id", it.id() },
+      } };
+      zypp::dumpAsXmlOn( *issue, it.title(), "title" );
+      zypp::dumpAsXmlOn( *issue, it.href(), "href" );
+    }
+    return str;
+  }
+
+  /** RNC: Print patch-update element */
+  inline std::ostream & xmlPrintPatchUpdateOn( std::ostream & str, const PoolItem & pi_r, const PatchHistoryData & patchHistoryData_r )
+  {
+    Patch::constPtr patch = pi_r->asKind<Patch>();
+
+    Patch::InteractiveFlags ignoreFlags = Patch::NoFlags;
+    if ( Zypper::instance().config().reboot_req_non_interactive )
+      ignoreFlags |= Patch::Reboot;
+    if ( LicenseAgreementPolicy::instance()._autoAgreeWithLicenses )
+      ignoreFlags |= Patch::License;
+
+    // write the node
+    xmlout::Node parent { str, "update", xmlout::Node::optionalContent, {
+      { "kind", "patch" },
+      { "name", patch->name () },
+      { "edition", patch->edition() },
+      { "arch", patch->arch() },
+      { "status", textPatchStatus( pi_r ) },
+      { "category", patch->category() },
+      { "severity", patch->severity() },
+      { "pkgmanager", asString( patch->restartSuggested() ) },
+      { "restart", asString( patch->rebootSuggested() ) },
+      { "interactive", asString( patch->interactiveWhenIgnoring(ignoreFlags) ) },
+    } };
+
+    if ( PatchHistoryData::value_type res { patchHistoryData_r[pi_r] }; res != PatchHistoryData::noData )
+    {
+      if ( res.second == pi_r.status().validate() )
+	dumpAsXmlOn( *parent, res.first, "status-since" );
+      else
+	// patch status was changed by a non zypp transaction (not mentioned in the history)
+	DBG << "PatchHistoryData " << res.second << " but " << pi_r << endl;
+    }
+    dumpAsXmlOn( *parent, patch->summary(), "summary" );
+    dumpAsXmlOn( *parent, patch->description(), "description" );
+    dumpAsXmlOn( *parent, patch->licenseToConfirm(), "license" );
+
+    if ( !patch->repoInfo().alias().empty() )
+    {
+      xmlout::Node( *parent, "source", xmlout::Node::optionalContent, {
+	{ "url", patch->repoInfo().url().asString() },
+	{ "alias", patch->repoInfo().alias() },
+      } );
+    }
+
+    dumpAsXmlOn( *parent, patch->timestamp(), "issue-date" );
+    xmlPrintPatchIssueListElementOn( *parent, patch->referencesBegin(), patch->referencesEnd() );
+    return str;
+  }
+
+  /** RNC: Print element containing patch-update-list */
+  template <typename TContainer>
+  inline std::ostream & xmlPrintPatchUpdateListOn( std::ostream & str, std::string nodename_r, TContainer container_r, const PatchHistoryData & patchHistoryData_r )
+  {
+    xmlout::Node parent { str, std::move(nodename_r), xmlout::Node::optionalContent };
+    for ( const auto & pi : container_r )
+      xmlPrintPatchUpdateOn( *parent, pi, patchHistoryData_r );
+    return str;
+  }
+
 } //namespace
 ///////////////////////////////////////////////////////////////////
 
@@ -200,7 +256,7 @@ namespace
 
     struct CategorySort
     {
-      bool operator()( const Patch::Category & lhs, const Patch::Category & rhs )
+      bool operator()( const Patch::Category & lhs, const Patch::Category & rhs ) const
       {
 	if ( lhs == rhs )
 	  return false;
@@ -347,19 +403,19 @@ namespace
       {
 	TableHeader hdr;
 	// translator: Table column header.
-	hdr << _("Category");
+	hdr << N_("Category");
 	// translator: Table column header.
-	if ( haveUPD )	hdr << _("Updatestack");
+	if ( haveUPD )	hdr << N_("Updatestack");
 	// translator: Table column header.
-	if ( havePAT )	hdr << _("Patches");
+	if ( havePAT )	hdr << N_("Patches");
 	// translator: Table column header.
-	if ( haveLCK )	hdr << ColorString( ctxtLocked, _("Locked") );
+	if ( haveLCK )	hdr << ColorString( ctxtLocked, N_("Locked") );
 	// translator: Table column header
 	// Used if stats collect data for more than one category name.
 	// Category    | Updatestack | Patches | Locked | Included categories
 	// ------------+-------------+---------+--------+---------------------
 	//  optional   | ...                      ..... | enhancement, feature
-	if ( haveAKA )	hdr << _("Included categories");
+	if ( haveAKA )	hdr << N_("Included categories");
 	tbl << std::move(hdr);
       }
 
@@ -383,14 +439,13 @@ namespace
 } // namespace
 ///////////////////////////////////////////////////////////////////
 
-void patch_check()
+void patch_check( bool updatestackOnly )
 {
   Zypper & zypper( Zypper::instance() );
   Out & out( zypper.out() );
   DBG << "patch check" << endl;
 
-  PatchCheckStats stats( zypper.globalOpts().exclude_optional_patches );
-  bool updatestackOnly = Zypper::instance().cOpts().count("updatestack-only");
+  PatchCheckStats stats( zypper.config().exclude_optional_patches );
   for_( it, God->pool().byKindBegin(ResKind::patch), God->pool().byKindEnd(ResKind::patch) )
   {
     const PoolItem & pi( *it );
@@ -414,45 +469,8 @@ void patch_check()
   { zypper.setExitCode( stats.security() ? ZYPPER_EXIT_INF_SEC_UPDATE_NEEDED : ZYPPER_EXIT_INF_UPDATE_NEEDED ); }
 }
 
-static void xml_print_patch( Zypper & zypper, const PoolItem & pi )
-{
-  Patch::constPtr patch = pi->asKind<Patch>();
-
-  cout << " <update ";
-  cout << "name=\"" << patch->name () << "\" ";
-  cout << "edition=\""  << patch->edition() << "\" ";
-  cout << "arch=\""  << patch->arch() << "\" ";
-  cout << "status=\""  << textPatchStatus( pi ) << "\" ";
-  cout << "category=\"" <<  patch->category() << "\" ";
-  cout << "severity=\"" <<  patch->severity() << "\" ";
-  cout << "pkgmanager=\"" << (patch->restartSuggested() ? "true" : "false") << "\" ";
-  cout << "restart=\"" << (patch->rebootSuggested() ? "true" : "false") << "\" ";
-
-  Patch::InteractiveFlags ignoreFlags = Patch::NoFlags;
-  if (zypper.globalOpts().reboot_req_non_interactive)
-    ignoreFlags |= Patch::Reboot;
-  if ( zypper.cOpts().count("auto-agree-with-licenses") )
-    ignoreFlags |= Patch::License;
-
-  cout << "interactive=\"" << (patch->interactiveWhenIgnoring(ignoreFlags) ? "true" : "false") << "\" ";
-  cout << "kind=\"patch\"";
-  cout << ">" << endl;
-  cout << "  <summary>" << xml::escape(patch->summary()) << "  </summary>" << endl;
-  cout << "  <description>" << xml::escape(patch->description()) << "</description>" << endl;
-  cout << "  <license>" << xml::escape(patch->licenseToConfirm()) << "</license>" << endl;
-
-  if ( !patch->repoInfo().alias().empty() )
-  {
-    cout << "  <source url=\"" << xml::escape(patch->repoInfo().url().asString());
-    cout << "\" alias=\"" << xml::escape(patch->repoInfo().alias()) << "\"/>" << endl;
-  }
-
-  cout << " </update>" << endl;
-}
-
-
 // returns true if NEEDED! restartSuggested() patches are available
-static bool xml_list_patches (Zypper & zypper)
+static bool xml_list_patches (Zypper & zypper, bool all_r, const PatchHistoryData & patchHistoryData_r )
 {
   const ResPool& pool = God->pool();
 
@@ -468,18 +486,17 @@ static bool xml_list_patches (Zypper & zypper)
   }
 
   unsigned patchcount = 0;
-  bool all = zypper.cOpts().count("all");
   for_( it, pool.byKindBegin(ResKind::patch), pool.byKindEnd(ResKind::patch) )
   {
-    if ( all || patchIsApplicable( *it ) )
+    if ( all_r || patchIsApplicable( *it ) )
     {
       const PoolItem & pi( *it );
       Patch::constPtr patch = pi->asKind<Patch>();
 
       // if updates stack patches are available, show only those
-      if ( all || !pkg_mgr_available || patchIsNeededRestartSuggested( pi ) )
+      if ( all_r || !pkg_mgr_available || patchIsNeededRestartSuggested( pi ) )
       {
-	xml_print_patch( zypper, pi );
+	xmlPrintPatchUpdateOn( cout, pi, patchHistoryData_r );
       }
     }
     ++patchcount;
@@ -494,7 +511,7 @@ static bool xml_list_patches (Zypper & zypper)
   {
     // close <update-list> and write <blocked-update-list> if not all
     cout << "</update-list>" << endl;
-    if ( ! all )
+    if ( ! all_r )
     {
     cout << "<blocked-update-list>" << endl;
     for_( it, pool.byKindBegin(ResKind::patch), pool.byKindEnd(ResKind::patch) )
@@ -504,7 +521,7 @@ static bool xml_list_patches (Zypper & zypper)
 	const PoolItem & pi( *it );
 	Patch::constPtr patch = pi->asKind<Patch>();
 	if ( ! patchIsNeededRestartSuggested( pi ) )
-	  xml_print_patch( zypper, pi );
+	  xmlPrintPatchUpdateOn( cout, pi, patchHistoryData_r );
       }
     }
     cout << "</blocked-update-list>" << endl;
@@ -516,58 +533,30 @@ static bool xml_list_patches (Zypper & zypper)
 
 // ----------------------------------------------------------------------------
 
-static void xml_list_updates(const ResKindSet & kinds)
+static void xml_list_updates(const ResKindSet & kinds, bool all_r )
 {
   Candidates candidates;
-  find_updates( kinds, candidates );
+  find_updates( kinds, candidates, all_r );
 
   for( const PoolItem & pi : candidates )
   {
-    cout << " <update ";
-    cout << "name=\"" << pi.name () << "\" " ;
-    cout << "edition=\""  << pi.edition() << "\" ";
-    cout << "arch=\""  << pi.arch() << "\" ";
-    cout << "kind=\"" << pi.kind() << "\" ";
-    // for packages show also the current installed version (bnc #466599)
-    {
-      const PoolItem & ipi( ui::Selectable::get(pi)->installedObj() );
-      if ( ipi )
-      {
-	if ( pi.edition() != ipi.edition() )
-	  cout << "edition-old=\""  << ipi.edition() << "\" ";
-	if ( pi.arch() != ipi.arch() )
-	  cout << "arch-old=\""  << ipi.arch() << "\" ";
-      }
-    }
-    cout << ">" << endl;
-    cout << "  <summary>" << xml::escape(pi.summary()) << "</summary>" << endl;
-    cout << "  <description>" << xml::escape(pi.description()) << "</description>" << endl;
-    cout << "  <license>" << xml::escape(pi.licenseToConfirm()) << "</license>" << endl;
-
-    if ( !pi.repoInfo().alias().empty() )
-    {
-        cout << "  <source url=\"" << xml::escape(pi.repoInfo().url().asString());
-        cout << "\" alias=\"" << xml::escape(pi.repoInfo().alias()) << "\"/>" << endl;
-    }
-
-    cout << " </update>" << endl;
+    xmlPrintOtherUpdateOn( cout, pi );
   }
 }
 
 // ----------------------------------------------------------------------------
 
 // returns true if NEEDED! restartSuggested() patches are available
-static bool list_patch_updates( Zypper & zypper )
+static bool list_patch_updates( Zypper & zypper, bool all_r, const PatchSelector &sel, const PatchHistoryData & historyData_r )
 {
   Table tbl;
-  FillPatchesTable intoTbl( tbl );
+  FillPatchesTable intoTbl( tbl, historyData_r );
 
   Table pmTbl; // only NEEDED! that affect packagemanager (restartSuggested()), they have priority
-  FillPatchesTable intoPMTbl( pmTbl );
+  FillPatchesTable intoPMTbl( pmTbl, historyData_r );
 
-  PatchCheckStats stats( zypper.globalOpts().exclude_optional_patches );
-  CliMatchPatch cliMatchPatch( zypper );
-  bool all = zypper.cOpts().count("all");
+  PatchCheckStats stats( zypper.config().exclude_optional_patches );
+  CliMatchPatch cliMatchPatch( zypper, sel._requestedPatchDates, sel._requestedPatchCategories, sel._requestedPatchSeverity );
 
   const ResPool& pool = God->pool();
   for_( it, pool.byKindBegin(ResKind::patch), pool.byKindEnd(ResKind::patch) )
@@ -583,9 +572,9 @@ static bool list_patch_updates( Zypper & zypper )
     if ( tostat )	// exclude cliMatchPatch filtered but include undisplayed ones
       stats.collect( pi );
 
-    if ( all || patchIsApplicable( pi ) )
+    if ( all_r || patchIsApplicable( pi ) )
     {
-      if ( ! all && patchIsNeededRestartSuggested( pi ) )
+      if ( ! all_r && patchIsNeededRestartSuggested( pi ) )
 	intoPMTbl( pi );
       else
 	intoTbl( pi );
@@ -638,13 +627,13 @@ static bool list_patch_updates( Zypper & zypper )
  * Find all available updates of given kind.
  */
 static void
-find_updates( const ResKind & kind, Candidates & candidates )
+find_updates( const ResKind & kind, Candidates & candidates, bool all_r )
 {
   const ResPool& pool = God->pool();
   DBG << "Looking for update candidates of kind " << kind << endl;
 
   // package updates
-  if (kind == ResKind::package && !Zypper::instance().cOpts().count("all"))
+  if (kind == ResKind::package && !all_r)
   {
     God->resolver()->doUpdate();
     ResPool::const_iterator
@@ -694,10 +683,10 @@ find_updates( const ResKind & kind, Candidates & candidates )
  * Find all available updates of given kinds.
  */
 void
-find_updates( const ResKindSet & kinds, Candidates & candidates )
+find_updates(const ResKindSet & kinds, Candidates & candidates , bool all_r)
 {
   for (ResKindSet::const_iterator kit = kinds.begin(); kit != kinds.end(); ++kit)
-    find_updates(*kit, candidates);
+    find_updates( *kit, candidates, all_r );
 
   if (kinds.empty())
     WAR << "called with empty kinds set" << endl;
@@ -723,8 +712,10 @@ std::string i18n_kind_updates(const ResKind & kind)
 
 // FIXME rewrite this function so that first the list of updates is collected and later correctly presented (bnc #523573)
 
-void list_updates(Zypper & zypper, const ResKindSet & kinds, bool best_effort)
+void list_updates(Zypper & zypper, const ResKindSet & kinds, bool best_effort, bool all_r, const PatchSelector &patchSel_r )
 {
+  PatchHistoryData patchHistoryData;	// commonly used by all tables
+
   if (zypper.out().type() == Out::TYPE_XML)
   {
     // TODO: go for XmlNode
@@ -747,7 +738,7 @@ void list_updates(Zypper & zypper, const ResKindSet & kinds, bool best_effort)
   if(it != localkinds.end())
   {
     if (zypper.out().type() == Out::TYPE_XML)
-      affects_pkgmgr = xml_list_patches(zypper);
+      affects_pkgmgr = xml_list_patches( zypper, all_r, patchHistoryData );
     else
     {
       if (kinds.size() > 1)
@@ -755,7 +746,7 @@ void list_updates(Zypper & zypper, const ResKindSet & kinds, bool best_effort)
         zypper.out().info("", Out::NORMAL, Out::TYPE_NORMAL);
         zypper.out().info(i18n_kind_updates(*it), Out::QUIET, Out::TYPE_NORMAL);
       }
-      affects_pkgmgr = list_patch_updates(zypper);
+      affects_pkgmgr = list_patch_updates( zypper, all_r, patchSel_r, patchHistoryData );
     }
     localkinds.erase(it);
   }
@@ -767,7 +758,7 @@ void list_updates(Zypper & zypper, const ResKindSet & kinds, bool best_effort)
   {
     if (!affects_pkgmgr)
     {
-      xml_list_updates(localkinds);
+      xml_list_updates( localkinds, all_r );
       cout << "</update-list>" << endl;		// otherwise closed in xml_list_patches
     }
     cout << "</update-status>" << endl;
@@ -784,24 +775,24 @@ void list_updates(Zypper & zypper, const ResKindSet & kinds, bool best_effort)
 
     // show repo only if not best effort or --from-repo set
     // on best_effort, the solver will determine the repo if we don't limit it to a specific one
-    bool hide_repo = best_effort || copts.count("repo");
+    bool hide_repo = best_effort || InitRepoSettings::instance()._repoFilter.size();
 
     // header
     TableHeader th;
     unsigned name_col;
     // TranslatorExplanation S stands for Status
-    th << _("S");
+    th << N_("S");
     if (!hide_repo)
-      th << _("Repository");
+      th << N_("Repository");
 
     name_col = th.cols();
-    th << _("Name");
+    th << N_("Name");
     // best_effort does not know version or arch yet
     if (!best_effort)
     {
       if (*it == ResKind::package)
-        th << table::EditionStyleSetter( tbl, _("Current Version") );
-      th << table::EditionStyleSetter( tbl, _("Available Version") ) << _("Arch");
+        th << table::EditionStyleSetter( tbl, N_("Current Version") );
+      th << table::EditionStyleSetter( tbl, N_("Available Version") ) << N_("Arch");
     }
 
     tbl << th;
@@ -811,12 +802,12 @@ void list_updates(Zypper & zypper, const ResKindSet & kinds, bool best_effort)
     ResPoolProxy uipool( ResPool::instance().proxy() );
 
     Candidates candidates;
-    find_updates( *it, candidates );
+    find_updates( *it, candidates, all_r );
 
     for ( const PoolItem & pi : candidates )
     {
       TableRow tr (cols);
-      tr << "v";
+      tr << computeStatusIndicator( pi );
       if (!hide_repo) {
         tr << pi.repoInfo().asUserString();
       }
@@ -855,21 +846,22 @@ void list_updates(Zypper & zypper, const ResKindSet & kinds, bool best_effort)
 }
 
 // ----------------------------------------------------------------------------
-
-void list_patches_by_issue( Zypper & zypper )
+void list_patches_by_issue( Zypper & zypper, bool all_r, const PatchSelector & sel_r )
 {
-  // --bz, --cve can't be used together with --issue; this case is ruled out
-  // in the initial arguments validation in Zypper.cc
-  Table issueMatchesTbl;
-  FillPatchesTableForIssue intoIssueMatchesTbl( issueMatchesTbl );
+  PatchHistoryData patchHistoryData;	// commonly used by all tables
 
-  CliScanIssues issues;
-  CliMatchPatch cliMatchPatch( zypper );
-  bool only_needed = !zypper.cOpts().count("all");
+  // Overall CLI filter
+  bool only_needed = !all_r;
+  CliMatchPatch cliMatchPatch( zypper,
+                               sel_r._requestedPatchDates,
+                               sel_r._requestedPatchCategories,
+                               sel_r._requestedPatchSeverity );
 
+
+  // pass1 finding PoolItems and their matching issues (pi,itype,iid)
   std::vector<const Issue*> pass2; // on the fly remember anyType issues for pass2
-
-  for ( const Issue & issue : issues )
+  std::map<PoolItem,std::map<std::string,std::set<std::string>>> iresult;
+  for ( const Issue & issue : sel_r._requestedIssues )
   {
     PoolQuery q;
     q.setMatchSubstring();
@@ -884,55 +876,40 @@ void list_patches_by_issue( Zypper & zypper )
       q.addAttribute( sat::SolvAttr::updateReferenceId, issue.id() );
       if ( issue.anyType() && issue.specificId() ) 				// remember for pass2
       {
-	q.addAttribute( sat::SolvAttr::updateReferenceType, issue.id() );	// bnc#941309: let '--issue-bugzilla' also match the type
+	q.addAttribute( sat::SolvAttr::updateReferenceType, issue.id() );	// bnc#941309: let '--issue=bugzilla' also match the type
 	pass2.push_back( &issue );
       }
     }
 
     for_( it, q.begin(), q.end() )
     {
-      PoolItem pi( *it );
-      Patch::constPtr patch = asKind<Patch>(pi);
+      PoolItem pi { *it };
 
       if ( only_needed && ! patchIsApplicable( pi ) )
 	continue;
 
-      if ( ! cliMatchPatch( patch ) )
+      if ( ! cliMatchPatch( pi ) )
       {
-	DBG << patch->ident() << " skipped. (not matching CLI filter)" << endl;
+	DBG << pi.ident() << " skipped. (not matching CLI filter)" << endl;
 	continue;
       }
 
-      // Print details about each match in that solvable:
-      // NOTE: If searching in BOTH updateReferenceId AND updateReferenceType,
-      // we may find matches in both but want to report the issue just once!
-      std::set<std::pair<std::string,std::string>> unifiedResults;
       for_( d, it.matchesBegin(), it.matchesEnd() )
       {
-        std::string itype = d->subFind( sat::SolvAttr::updateReferenceType ).asString();
-
+	std::string itype { d->subFind( sat::SolvAttr::updateReferenceType ).asString() };
 	if ( issue.specificType() && itype != issue.type() )
 	  continue;	// assert correct type of specific IDs
-
 	// remember....
-	unifiedResults.insert( std::make_pair( std::move(itype), d->subFind( sat::SolvAttr::updateReferenceId ).asString() ) );
-      }
-      // print remembered...
-      for ( auto & p : unifiedResults )
-      {
-	intoIssueMatchesTbl( pi, std::move(p.first), std::move(p.second) );
+	iresult[std::move(pi)][std::move(itype)].insert( d->subFind( sat::SolvAttr::updateReferenceId ).asString() );
       }
     }
   }
 
-  // pass2: look for matches in patch summary/description
-  //
-  Table summaryMatchesTbl;
-  FillPatchesTable intoSummaryMatchesTbl( summaryMatchesTbl );
-
-  for ( const Issue* _issue : pass2 )
+  //pass2 (summary/description)
+  std::vector<PoolItem> dresult;	// query will assert unique entries
+  for ( const Issue * _issue : pass2 )
   {
-    const Issue & issue( *_issue );
+    const Issue & issue { *_issue };
     PoolQuery q;
     q.setMatchSubstring();
     q.setCaseSensitive( false );
@@ -942,64 +919,86 @@ void list_patches_by_issue( Zypper & zypper )
 
     for_( it, q.begin(), q.end() )
     {
-      PoolItem pi( *it );
-      Patch::constPtr patch = asKind<Patch>(pi);
+      PoolItem pi { *it };
 
       if ( only_needed && ! patchIsApplicable( pi ) )
 	continue;
 
-      if ( ! cliMatchPatch( patch ) )
+      if ( ! cliMatchPatch( pi ) )
       {
-	DBG << patch->ident() << " skipped. (not matching CLI filter)" << endl;
+	DBG << pi.ident() << " skipped. (not matching CLI filter)" << endl;
 	continue;
       }
 
-      intoSummaryMatchesTbl( pi );
-//	issueMatchesTbl.rows().back().addDetail( str::Str() << "    " << attr.inSolvAttr() << "\t\"" << attr.asString() << "\"" );
-      //! \todo could show a highlighted match with a portion of surrounding
-      //! text. Needs case-insensitive find.
+      if ( ! iresult.count( pi ) )
+      { dresult.push_back( pi ); }
     }
   }
 
+  ///////////////////////////////////////////////////////////////////
   // print result
-  if ( issueMatchesTbl.empty() && summaryMatchesTbl.empty() )
-  {  zypper.out().info(_("No matching issues found.")); }
+  if (zypper.out().type() == Out::TYPE_XML)
+  {
+    xmlout::Node parent { cout, "list-patches-byissue", xmlout::Node::optionalContent };
+    xmlPrintPatchUpdateListOn( *parent, "issue-matches", zypp_pending::make_map_key_Iterable( iresult ), patchHistoryData );
+    xmlPrintPatchUpdateListOn( *parent, "description-matches", dresult, patchHistoryData );
+  }
   else
   {
-    if ( !issueMatchesTbl.empty() )
+    // iresult to table
+    Table issueMatchesTbl;
+    FillPatchesTableForIssue intoIssueMatchesTbl( issueMatchesTbl, patchHistoryData );
+    for ( const auto & res : iresult )
     {
-      if ( !pass2.empty() )
+      const PoolItem & pi { res.first };
+      for ( const auto & ires : res.second )
       {
-        cout << endl;
-        zypper.out().info(_("The following matches in issue numbers have been found:"));
+	const std::string & itype { ires.first };
+	for ( const std::string & iid : ires.second )
+	{ intoIssueMatchesTbl( pi, itype, iid ); }
       }
-      issueMatchesTbl.sort(); // use default sort
-      cout << endl << issueMatchesTbl;
     }
 
-    if ( !summaryMatchesTbl.empty() )
+    // dresult to table
+    Table descrMatchesTbl;
+    FillPatchesTable intoDescrMatchesTbl( descrMatchesTbl, patchHistoryData );
+    for ( const auto & pi : dresult )
+    {
+      intoDescrMatchesTbl( pi );
+    }
+
+
+    if ( issueMatchesTbl.empty() && descrMatchesTbl.empty() )
+    {  zypper.out().info(_("No matching issues found.")); }
+    else
     {
       if ( !issueMatchesTbl.empty() )
-      { cout << endl; }
-      zypper.out().info(_( "Matches in patch descriptions of the following patches have been found:"));
-      summaryMatchesTbl.sort(); // use default sort
-      cout << endl << summaryMatchesTbl;
+      {
+	zypper.out().gap();
+	zypper.out().info(_("The following matches in issue numbers have been found:"));
+
+	issueMatchesTbl.sort(); // use default sort
+	zypper.out().gap();
+	cout << issueMatchesTbl;
+      }
+
+      if ( !descrMatchesTbl.empty() )
+      {
+	zypper.out().gap();
+	zypper.out().info(_( "Matches in patch descriptions of the following patches have been found:"));
+
+	descrMatchesTbl.sort(); // use default sort
+	zypper.out().gap();
+	cout << descrMatchesTbl;
+      }
     }
   }
 }
 
 // ----------------------------------------------------------------------------
 
-void mark_updates_by_issue( Zypper & zypper )
+void mark_updates_by_issue( Zypper & zypper, const std::set<Issue> &issues, SolverRequester::Options srOpts )
 {
-  CliScanIssues issues;
-
-  SolverRequester::Options srOpts;
-  srOpts.force = zypper.cOpts().count("force");
-  srOpts.skip_interactive = zypper.cOpts().count("skip-interactive");
-  srOpts.skip_optional_patches = zypper.globalOpts().exclude_optional_patches;
-  srOpts.cliMatchPatch = CliMatchPatch( zypper );
-
   for ( const Issue & issue : issues )
   {
     PoolQuery q;
